@@ -1,9 +1,10 @@
 package Test::Smoke;
 use strict;
 
-# $Id: Smoke.pm 484 2003-10-20 05:46:22Z abeltje $
-use vars qw( $VERSION $conf @EXPORT );
-$VERSION = '1.18.10';
+# $Id: Smoke.pm 873 2005-07-21 16:54:14Z abeltje $
+use vars qw( $VERSION $REVISION $conf @EXPORT );
+$VERSION  = '1.19_71';
+$REVISION = __get_ts_patchlevel();
 
 use base 'Exporter';
 @EXPORT  = qw( $conf &read_config &run_smoke );
@@ -119,10 +120,10 @@ sub run_smoke {
 
     my $patch = Test::Smoke::Util::get_patch( $conf->{ddir} );
 
-    exists $Config{ldlibpthname} && $Config{ldlibpthname} and
-        $ENV{ $Config{ldlibpthname} } ||= '',
-        substr( $ENV{ $Config{ldlibpthname} }, 0, 0)  = 
-            "$conf->{ddir}$Config{path_sep}";
+#    exists $Config{ldlibpthname} && $Config{ldlibpthname} and
+#        $ENV{ $Config{ldlibpthname} } ||= '',
+#        substr( $ENV{ $Config{ldlibpthname} }, 0, 0)  =
+#            "$conf->{ddir}$Config{path_sep}";
 
     my $logfile = File::Spec->catfile( $conf->{ddir}, 'mktest.out' );
     my $BuildCFG = $continue 
@@ -143,13 +144,13 @@ sub run_smoke {
     $conf->{v} && $conf->{defaultenv} and
         $smoker->tty( "Running smoke tests without \$ENV{PERLIO}\n" );
 
+    chdir $conf->{ddir} or die "Cannot chdir($conf->{ddir}): $!";
     unless ( $continue ) {
-        $smoker->_make( "-i distclean 2>/dev/null" );
+        $smoker->make_distclean( );
         $smoker->ttylog( "Smoking patch $patch\n" ); 
         do_manifest_check( $conf->{ddir}, $smoker );
     }
 
-    chdir $conf->{ddir} or die "Cannot chdir($conf->{ddir}): $!";
     foreach my $this_cfg ( $BuildCFG->configurations ) {
         $smoker->mark_out; $smoker->mark_in;
         if ( skip_config( $this_cfg ) ) {
@@ -171,13 +172,32 @@ sub run_smoke {
    };
 }
 
+=item __get_ts_patchlevel( )
+
+Read the contents of F<.patch>.
+
+=cut
+
+use FindBin;
+use File::Spec::Functions;
+
+sub __get_ts_patchlevel {
+    my( $rev ) = q$Rev: 873 $ =~ /(\d+)/;
+    my $dotpatch = catfile $FindBin::Bin, '.patch';
+    local *DOTPATCH;
+    open DOTPATCH, "< $dotpatch" or return $rev;
+    chomp( my $plevel = <DOTPATCH> );
+    close DOTPATCH;
+    return $plevel > $rev ? $plevel : $rev;
+}
+
 1;
 
 =back
 
 =head1 REVISION
 
-$Id: Smoke.pm 484 2003-10-20 05:46:22Z abeltje $
+$Id: Smoke.pm 873 2005-07-21 16:54:14Z abeltje $
 
 =head1 COPYRIGHT
 
