@@ -1,29 +1,31 @@
 package Test::Smoke::Util;
 use strict;
 
-# $Id: Util.pm 412 2003-08-27 13:07:23Z abeltje $
+# $Id: Util.pm 871 2005-07-21 16:26:52Z abeltje $
 use vars qw( $VERSION @EXPORT @EXPORT_OK );
-$VERSION = '0.23';
+$VERSION = '0.38';
 
 use base 'Exporter';
 @EXPORT = qw( 
-    &Configure_win32 
+    &Configure_win32
     &get_cfg_filename &get_config
-    &check_MANIFEST
     &get_patch
     &skip_config &skip_filter
 );
 
-@EXPORT_OK = qw( 
+@EXPORT_OK = qw(
+    &grepccmsg
     &get_ncpu &get_smoked_Config &parse_report_Config 
     &get_regen_headers &run_regen_headers
-    &calc_timeout 
+    &calc_timeout &time_in_hhmm
     &do_pod2usage
+    &set_vms_rooted_logical
 );
 
 use Text::ParseWords;
-use File::Spec;
+use File::Spec::Functions;
 use File::Find;
+use Cwd;
 
 =head1 NAME
 
@@ -122,6 +124,10 @@ installation.
 Set the base directory for the C compiler.
 B<$(CCHOME)\bin> still needs to be in the path!
 
+=item * B<-DIS_WIN95>
+
+sets IS_WIN95 to 'define' to indicate this is Win9[58]
+
 =item * B<-DCRYPT_SRC=...>
 
 The file to use as source for des_fcrypt()
@@ -151,7 +157,9 @@ my %win32_makefile_map = (
 
 sub Configure_win32 {
     my($command, $win32_maker, @args ) = @_;
-    $win32_maker ||= 'nmake';
+    $win32_maker ||= 'nmake'; $win32_maker = lc $win32_maker;
+    my $is_dmake = $win32_maker eq 'dmake';
+    my $is_nmake = $win32_maker eq 'nmake';
 
     local $_;
     my %opt_map = (
@@ -160,6 +168,7 @@ sub Configure_win32 {
 	"-Duseperlio"		=> "USE_PERLIO",
 	"-Dusemultiplicity"	=> "USE_MULTI",
 	"-Duseimpsys"		=> "USE_IMP_SYS",
+	"-Uuseimpsys"		=> "USE_IMP_SYS",
         "-Dusemymalloc"         => "PERL_MALLOC",
         "-Duselargefiles"       => "USE_LARGE_FILES",
 	"-DDEBUGGING"		=> "USE_DEBUGGING",
@@ -172,6 +181,7 @@ sub Configure_win32 {
         "-Dgcc_v3_2"            => "USE_GCC_V3_2",
         "-Dbccold"              => "BCCOLD",
         "-DCCHOME"              => "CCHOME",
+        "-DIS_WIN95"            => "IS_WIN95",
         "-DCRYPT_SRC"           => "CRYPT_SRC",
         "-DCRYPT_LIB"           => "CRYPT_LIB",
     );
@@ -197,33 +207,43 @@ sub Configure_win32 {
         USE_GCC_V3_2    => 0,
         BCCOLD          => 0,
         CCHOME          => undef,
+        IS_WIN95        => 0,
         CRYPT_SRC       => undef,
         CRYPT_LIB       => undef,
     );
-#    my $def_re = qr/((?:(?:PERL|USE)_\w+)|BCCOLD)/;
-    my $def_re = '((?:(?:PERL|USE)_\w+)|BCCOLD)';
+#    my $def_re = qr/((?:(?:PERL|USE|IS)_\w+)|BCCOLD)/;
+    my $def_re = '((?:(?:PERL|USE|IS)_\w+)|BCCOLD)';
     my @w32_opts = grep ! /^$def_re/, keys %opts;
     my $config_args = join " ", 
-        grep /^-D[a-z_]+/, quotewords( '\s+', 1, $command );
+        grep /^-[DU][a-z_]+/, quotewords( '\s+', 1, $command );
     push @args, "config_args=$config_args";
 
     my @buildopt;
     $command =~ m{^\s*\./Configure\s+(.*)} or die "unable to parse command";
-    foreach ( quotewords( '\s+', 1, $1) ) {
+    my $cmdln = $1;
+    foreach ( quotewords( '\s+', 1, $cmdln ) ) {
 	m/^-[des]{1,3}$/ and next;
 	m/^-Dusedevel$/  and next;
-        if ( /^-Accflags=(['"])?(.+)\1/ ) { #emacs' syntaxhighlite
+        if ( /^-Accflags=(['"]?)(.+)\1/ ) { #emacs' syntaxhighlite
            push @buildopt, $2;
            next;
         }
-        my( $option, $value ) = /^(-D\w+)(?:=(.+))?$/;
+        my( $option, $value ) = /^(-[DU]\w+)(?:=(.+))?$/;
 	die "invalid option '$_'" unless exists $opt_map{$option};
 	$opts{$opt_map{$option}} = $value ? $value : 1;
+        $option =~ /^-U/ and $opts{$opt_map{$option}} = 0;
     }
 
     # If you set one, we do all, so you can have fork()
-    if ( $opts{USE_MULTI} || $opts{USE_ITHREADS} || $opts{USE_IMP_SYS} ) {
-        $opts{USE_MULTI} = $opts{USE_ITHREADS} = $opts{USE_IMP_SYS} = 1;
+    # unless you set -Uuseimpsys
+    unless ( $cmdln =~ /-Uuseimpsys\b/ ) {
+        if ( $opts{USE_MULTI} || $opts{USE_ITHREADS} || $opts{USE_IMP_SYS} ) {
+            $opts{USE_MULTI} = $opts{USE_ITHREADS} = $opts{USE_IMP_SYS} = 1;
+        }
+    } else {
+        if ( $opts{USE_MULTI} || $opts{USE_ITHREADS} ) {
+            $opts{USE_MULTI} = $opts{USE_ITHREADS} = 1;
+        }
     }
 
     # If you -Dgcc_v3_2 you 'll *want* CCTYPE = GCC
@@ -242,8 +262,10 @@ sub Configure_win32 {
     my $donot_change = 0;
     while (<ORG>) {
         if ( $donot_change ) {
+            # need to help the Win95 build
+            $is_dmake and s/\b$win32_makefile_map{ $win32_maker }\b/smoke.mk/;
             if (m/^\s*CFG_VARS\s*=/) {
-                my( $extra_char, $quote ) = $win32_maker =~ /\bnmake\b/ 
+                my( $extra_char, $quote ) = $is_nmake
                     ? ( "\t", '"' ) : ("~", "" );
                 $_ .= join "", map "\t\t$quote$_$quote\t${extra_char}\t\\\n", 
                                    grep /\w+=/, @args;
@@ -283,6 +305,42 @@ sub Configure_win32 {
     return $out;
 } # Configure_win32
 
+=item set_vms_rooted_logical( $logical, $dir )
+
+This will set a VMS rooted logical like:
+
+    define/translation=concealed $logical $dir
+
+=cut
+
+sub set_vms_rooted_logical {
+    my( $logical, $dir ) = @_;
+    return unless $^O eq 'VMS';
+
+    my $cwd = cwd();
+    $dir ||= $cwd;
+
+    chdir $dir or die "Cannot chdir($dir): $!";
+
+    # On older systems we might exceed the 8-level directory depth limit
+    # imposed by RMS.  We get around this with a rooted logical, but we
+    # can't create logical names with attributes in Perl, so we do it
+    # in a DCL subprocess and put it in the job table so the parent sees it.
+
+    open TSBRL, '> tsbuildrl.com' or die "Error creating DCL-file; $!";
+
+    print TSBRL <<COMMAND;
+\$ $logical = F\$PARSE("SYS\$DISK:[]",,,,"NO_CONCEAL")-".][000000"-"]["-"].;"+".]"
+\$ DEFINE/JOB/NOLOG/TRANSLATION=CONCEALED $logical '$logical'
+COMMAND
+    close TSBRL;
+
+    my $result = system '@tsbuildrl.com';
+    1 while unlink 'tsbuildrl.com';
+    chdir $cwd;
+    return $result == 0;
+}
+
 =item get_cfg_filename( )
 
 C<get_cfg_filename()> tries to find a B<cfg file> and returns it.
@@ -304,6 +362,117 @@ sub get_cfg_filename {
     return undef;
 }
 
+=item grepccmsg( $cc, $logfile, $verbose )
+
+This is a port of Jarkko Hietaniemi's grepccerr script.
+
+=cut
+
+sub grepccmsg {
+    my( $cc, $logfile, $verbose ) = @_;
+    defined $logfile or return;
+    $cc ||= 'gcc';
+    my %OS2PAT = (
+        'aix' => 
+            # "foo.c", line n.c: pppp-qqq (W) ...error description...
+            # "foo.c", line n.c: pppp-qqq (S) ...error description...
+            '(^".+?", line \d+\.\d+: \d+-\d+ \([WS]\) .+?$)',
+
+        'dec_osf' =>
+            # DEC OSF/1, Digital UNIX, Tru64 (notice also VMS)
+            # cc: Warning: foo.c, line nnn: ...error description...(error_tag)
+            #     ...error line...
+            # ------^
+            # cc: Error: foo.c, line nnn: ...error description... (error_tag)
+            #     ...error line...
+            # ------^
+            '(^cc: (?:Warning|Error): .+?^-*\^$)',
+
+       'hpux' =>
+            # cc: "foo.c"" line nnn: warning ppp: ...error description...
+            # cc: "foo.c"" line nnn: error ppp: ...error description...
+            '(^cc: ".+?", line \d+: (?:warning|error) \d+: .+?$)',
+
+        'irix' =>
+            # cc-pppp cc: WARNING File = foo.c, Line = nnnn
+            # ...error description...
+            # 
+            # ...error line...
+            #   ^
+            # cc-pppp cc: ERROR File = foo.c, Line = nnnn
+            # ...error description...
+            # 
+            # ...error line...
+            #   ^
+            '^(cc-\d+ cc: (?:WARNING|ERROR) File = .+?, ' .
+            'Line = \d+.+?^\s*\^$)',
+
+        'solaris' =>
+            # "foo.c", line nnn: warning: ...error description...
+            # "foo.c", line nnn: warning: ...:
+            #         ...error description...
+            # "foo.c", line nnn: syntax error ...
+            '(^".+?", line \d+: ' .
+            '(?:warning: (?:(?:.+?:$)?.+?$)|syntax error.+?$))',
+
+        'vms' => # same compiler as Tru64, different message syntax
+            #     ...error line...
+            # ......^
+            # %CC-W-MESSAGEID, ...error description...
+            # at line number nnn in file foo.c
+            '(^\n.+?\n^\.+?\^\n^\%CC-(?:I|W|E|F)-\w+, ' .
+            '.+?\nat line number \d+ in file \S+?$)',
+
+        'gcc' =>
+            # foo.c: In function `foo':
+            # foo.c:nnn: warning: ...
+            # foo.c: In function `foo':
+            # foo.c:nnn:ppp: warning: ...
+                # Sometimes also the column is mentioned.
+            # foo.c: In function `foo':
+            # foo.c:nnn: error: ...
+            '(^(?-s:.+?):(?: In function .+?:$|\d+(?:\:\d+)?: ' .
+            '(?:warning|error): .+?$))',
+
+        'mswin32' => # MSVC(?:60)*
+            # foo.c : error LNKnnn: error description
+            # full\path\to\fooc.c : fatal error LNKnnn: error description
+            # foo.c(nnn) : warning Cnnn: warning description
+            '(^(?!NMAKE)(?-s:.+?) : (?-s:.+?)\d+: .+?$)',
+
+    );
+    exists $OS2PAT{ lc $cc } or $cc = 'gcc';
+    my $pat = $OS2PAT{ lc $cc };
+
+    my( $indx, %error ) = ( 1 );
+    my $smokelog = '';
+    if ( open my $logfh, "< $logfile" ) {
+        $verbose and print "Reading logfile '$logfile'\n";
+        local $/;
+        $smokelog = <$logfh>;
+        close $logfh;
+        $verbose and print "Pattern($cc): /$pat/\n";
+    } else {
+        $verbose and print "Skipping '$logfile' '$!'\n";
+        $error{ "Couldn't examine '$logfile' for compiler warnings." } = 1;
+    }
+
+    $error{ $1 } ||= $indx++ while $smokelog =~ /$pat/smg;
+
+    # I need to think about this IRIX/$Config{cc} thing
+#    if ($cc eq 'irix') {
+#        if ($Config{cc} =~ /-n32|-64/) {
+#    	     delete @error{ grep { /cc-(?:1009|1110|1047) / } keys %error };
+#        }
+#    }
+
+    my @errors = map {
+        chomp; $_;
+    } sort { $error{ $a } <=> $error{ $b } } keys %error;
+
+    return wantarray ? @errors : \@errors;
+}
+
 =item get_config( $filename )
 
 Read and parse the configuration from file, or return the default
@@ -395,40 +564,6 @@ sub get_config {
     return @cnf_stack;
 }
 
-=item check_MANIFEST( $path )
-
-Read C<MANIFEST> from C<$path> and check against the actual directory
-contents.
-
-C<check_MANIFEST()> returns a hashref all keys are suspicious files, 
-their value tells wether it should not be there (false) or is missing (true).
-
-=cut
-
-sub check_MANIFEST {
-    my( $path ) = @_;
-    my $mani_name = File::Spec->catfile( $path, 'MANIFEST' );
-    my %MANIFEST;
-    if (open MANIFEST, "< $mani_name") {
-        # We've done no tests yet, and we've started after the rsync --delete
-        # Now check if I'm in sync
-        %MANIFEST = ( ".patch" => 1, map { s/\s.*//s; $_ => 1 } <MANIFEST>);
-        find (sub {
-            -d and return;
-            m/^mktest\.(log|out)$/ and return;
-            my $f = $File::Find::name;
-            $f =~ s:^\Q$path\E/?::;
-            if (exists $MANIFEST{$f}) {
-                delete $MANIFEST{$f};
-                return;
-            }
-            $MANIFEST{$f} = 0;
-        }, $path);
-    }
-    close MANIFEST;
-    return \%MANIFEST;
-}
-
 =item get_patch( [$ddir] )
 
 Try to find the patchlevel, look for B<.patch> or try to get it from
@@ -488,6 +623,15 @@ sub version_from_patchlevel_h {
     if ( open PATCHLEVEL, "< $file" ) {
         my $patchlevel = do { local $/; <PATCHLEVEL> };
         close PATCHLEVEL;
+
+        if ( $patchlevel =~ /^#define PATCHLEVEL\s+(\d+)/m ) {
+            # Also support perl < 5.6
+            $version = sprintf "%03u", $1;
+            $subversion = $patchlevel =~ /^#define SUBVERSION\s+(\d+)/m
+                ? sprintf "%02u", $1 : '??';
+            return "$revision.$version$subversion";
+        }
+
         $revision   = $patchlevel =~ /^#define PERL_REVISION\s+(\d+)/m 
                     ? $1 : '?';
         $version    = $patchlevel =~ /^#define PERL_VERSION\s+(\d+)/m
@@ -551,7 +695,7 @@ sub get_ncpu {
 	};
 
         /irix/i && do {
-            my @output = `hinv -c processor`;
+            my @output = grep /\s+processors?$/i => `hinv -c processor`;
             $cpus = (split " ", $output[0])[0];
             last OS_CHECK;
         };
@@ -578,6 +722,12 @@ sub get_ncpu {
             last OS_CHECK;
         };
 
+        /vms/i && do {
+            my @output = grep /CPU \d+ is in RUN state/ => `show cpu/active`;
+            $cpus = @output ? scalar @output : '';
+            last OS_CHECK;
+        };
+
         $cpus = "";
         require Carp;
         Carp::carp "get_ncpu: unknown operationg system";
@@ -598,39 +748,72 @@ fallback information from C<POSIX::uname()> and F<patchlevel.h>.
 =cut
 
 sub get_smoked_Config {
-    my $dir = shift;
-    my %Config = map { ( lc $_ => "" ) } @_;
+    my( $dir, @fields ) = @_;
+    my %Config = map { ( lc $_ => undef ) } @fields;
 
-    my $perl_Config_pm = File::Spec->catfile ($dir, "lib", "Config.pm");
-    my $perl_config_sh = File::Spec->catfile( $dir, 'config.sh' );
+    my $perl_Config_heavy = catfile ($dir, "lib", "Config_heavy.pl");
+    my $perl_Config_pm    = catfile ($dir, "lib", "Config.pm");
+    my $perl_config_sh    = catfile( $dir, 'config.sh' );
     local *CONF;
-    if ( open CONF, "< $perl_Config_pm" ) {
+    if ( open CONF, "< $perl_Config_heavy" ) {
 
         while (<CONF>) {
-            if (m/^(our|my) \$[cC]onfig_[sS][hH](.*) = <<'!END!';/..m/^!END!/){
+            if ( m/^(?:
+                       (?:our|my)\ \$[cC]onfig_[sS][hH].*
+                    |
+                       \$_
+                    )\ =\ <<'!END!';/x..m/^!END!/){
                 m/!END!(?:';)?$/      and next;
                 m/^([^=]+)='([^']*)'/ or next;
                 exists $Config{lc $1} and $Config{lc $1} = $2;
             }
         }
         close CONF;
-    } elsif ( open CONF, "< $perl_config_sh" ) {
-        while ( <CONF> ) {
-            m/^([^=]+)='([^']*)'/ or next; # '
-            exists $Config{ lc $1} and $Config{ lc $1 } = $2;
+    }
+    my %conf2 = map {
+        ( $_ => undef )
+    } grep !defined $Config{ $_ } => keys %Config;
+    if ( open CONF, "< $perl_Config_pm" ) {
+
+        while (<CONF>) {
+            if ( m/^(?:
+                       (?:our|my)\ \$[cC]onfig_[sS][hH].*
+                    |
+                       \$_
+                    )\ =\ <<'!END!';/x..m/^!END!/){
+                m/!END!(?:';)?$/      and next;
+                m/^([^=]+)='([^']*)'/ or next;
+                exists $conf2{lc $1} and $Config{lc $1} = $2;
+            }
         }
         close CONF;
-    } else { 
+    }
+    %conf2 = map {
+        ( $_ => undef )
+    } grep !defined $Config{ $_ } => keys %Config;
+    if ( open CONF, "< $perl_config_sh" ) {
+        while ( <CONF> ) {
+            m/^([^=]+)='([^']*)'/ or next; # '
+            exists $conf2{ $1} and $Config{ lc $1 } = $2;
+        }
+        close CONF;
+    }
+    %conf2 = map {
+        ( $_ => undef )
+    } grep !defined $Config{ $_ } => keys %Config;
+    if ( keys %conf2 ) { 
         # Fall-back values from POSIX::uname() (not reliable)
         require POSIX;
         my( $osname, undef, $osvers, undef, $arch) = POSIX::uname();
-        $Config{osname} = lc $osname if exists $Config{osname};
-        $Config{osvers} = lc $osvers if exists $Config{osvers};
-        $Config{archname} = lc $arch if exists $Config{archname};
-        $Config{version} = version_from_patchlevel_h( $dir )
-            if exists $Config{version};
+        $Config{osname}   = lc $osname if exists $conf2{osname};
+        $Config{osvers}   = lc $osvers if exists $conf2{osvers};
+        $Config{archname} = lc $arch   if exists $conf2{archname};
+        $Config{version}  = version_from_patchlevel_h( $dir )
+            if exists $conf2{version};
     }
 
+    # There should be no under-bars in perl versions!
+    exists $Config{version} and $Config{version} =~ s/_/./g;
     return %Config;
 }
 
@@ -649,10 +832,10 @@ sub parse_report_Config {
     my $version  = $report =~ /^Automated.*for (.+) patch/ ? $1 : '';
     my $plevel   = $report =~ /^Automated.*patch (\d+(?:\.\d+\.\d+-RC\d+)?)/
         ? $1 : '';
-    my $osname   = $report =~ /\bon (.*) - / ? $1 : '';
-    my $osvers   = $report =~ /\bon .* - (.*)/? $1 : '';
+    my $osname   = $report =~ /\bon\s+(.*) - / ? $1 : '';
+    my $osvers   = $report =~ /\bon\s+.* - (.*)/? $1 : '';
     $osvers =~ s/\s+\(.*//;
-    my $archname = $report =~ / \((.*)\)/ ? $1 : '';
+    my $archname = $report =~ /:.* \((.*)\)/ ? $1 : '';
     my $summary  = $report =~ /^Summary: (.*)/m ? $1 : '';
 
     return ( $version, $plevel, $osname, $osvers, $archname, $summary );
@@ -714,7 +897,7 @@ sub run_regen_headers {
     return 1;
 }
 
-=item calc_timeout( $killtime )
+=item calc_timeout( $killtime[, $from] )
 
 C<calc_timeout()> calculates the timeout in seconds. 
 C<$killtime> can be one of two formats:
@@ -728,30 +911,62 @@ to translate that to seconds.
 
 =item B<hh:mm>
 
-This format represents a clock time (localtime).
-Calculate minutes from midnight for both C<$killtime> and C<localtime()>,
-and get the difference.
+This format represents a clock time (localtime).  Calculate minutes
+from midnight for both C<$killtime> and C<localtime($from)>, and get
+the difference. If C<$from> is omitted, C<time()> is used.
+
+If C<$killtime> is the actual time, the timeout will be 24 hours!
 
 =back
 
 =cut
 
 sub calc_timeout {
-    my( $killtime ) = @_;
+    my( $killtime, $from ) = @_;
     my $timeout = 0;
     if ( $killtime =~ /^\+(\d+):([0-5]?[0-9])$/ ) {
         $timeout = 60 * (60 * $1 + $2 );
     } elsif ( $killtime =~ /^((?:[0-1]?[0-9])|(?:2[0-3])):([0-5]?[0-9])$/ ) {
+        defined $from or $from = time;
         my $time_min = 60 * $1 + $2;
-        my( $now_m, $now_h ) = (localtime)[1, 2];
+        my( $now_m, $now_h ) = (localtime $from)[1, 2];
         my $now_min = 60 * $now_h + $now_m;
         my $kill_min = $time_min - $now_min;
-        $kill_min += 60 * 24 if $kill_min < 0;
+        $kill_min += 60 * 24 if $kill_min <= 0;
         $timeout = 60 * $kill_min;
     }
     return $timeout;
 }
 
+=item time_in_hhmm( $diff )
+
+Create a string telling elapsed time in days, hours, minutes, seconds
+from the number of seconds.
+
+=cut
+
+sub time_in_hhmm {
+    my $diff = shift;
+
+    # Only show decimal point for diffs < 5 minutes
+    my $digits = $diff =~ /\./ ? $diff < 5*60 ? 3 : 0 : 0;
+    my $days = int( $diff / (24*60*60) );
+    $diff -= 24*60*60 * $days;
+    my $hour = int( $diff / (60*60) );
+    $diff -= 60*60 * $hour;
+    my $mins = int( $diff / 60 );
+    $diff -=  60 * $mins;
+    $diff = sprintf "%.${digits}f", $diff;
+
+    my @parts;
+    $days and push @parts, sprintf "%d day%s",   $days, $days == 1 ? "" : 's';
+    $hour and push @parts, sprintf "%d hour%s",  $hour, $hour == 1 ? "" : 's';
+    $mins and push @parts, sprintf "%d minute%s",$mins, $mins == 1 ? "" : 's';
+    $diff && !$days && !$hour and push @parts, "$diff seconds";
+
+    return join " ", @parts;
+}
+
 =item do_pod2man( %pod2usage_options )
 
 If L<Pod::Usage> is there then call its C<pod2usage()>.
@@ -791,7 +1006,13 @@ sub skip_config {
     my( $config ) = @_;
 
     my $skip = $config->has_arg(qw( -Uuseperlio -Dusethreads )) ||
-               $config->has_arg(qw( -Uuseperlio -Duseithreads ));
+               $config->has_arg(qw( -Uuseperlio -Duseithreads )) ||
+               ( $^O eq 'MSWin32' && 
+               (( $config->has_arg(qw( -Duseithreads -Dusemymalloc )) &&
+                !$config->has_arg( '-Uuseimpsys' ) ) ||
+               ( $config->has_arg(qw( -Dusethreads -Dusemymalloc )) &&
+                !$config->has_arg( '-Uuseimpsys' ) ))
+               );
     return $skip;
 }
 
@@ -826,6 +1047,7 @@ sub skip_filter {
 #    m,^\s+$testdir/, ||
     m,^sh mv-if-diff\b, ||
     m,File \S+ not changed, ||
+    m,^(not\s+)?ok\s+\d+\s+[-#]\s+(?i:skip\S*[: ]),i ||
     # cygwin
     m,^dllwrap: no export definition file provided, ||
     m,^dllwrap: creating one. but that may not be what you want, ||
@@ -840,6 +1062,7 @@ sub skip_filter {
     m,^(   )?### , ||
     # Clean up Win32's output
     m,^(?:\.\.[/\\])?[\w/\\-]+\.*ok$, ||
+    m,^(?:\.\.[/\\])?[\w/\\-]+\.*ok\s+\d+(\.\d+)?s$, ||
     m,^(?:\.\.[/\\])?[\w/\\-]+\.*ok\,\s+\d+/\d+\s+skipped:, ||
     m,^(?:\.\.[/\\])?[\w/\\-]+\.*skipped[: ], ||
     m,^\t?x?copy , ||
